//Last modified 3/23/8
//*****************************************************************************************
//Description: This manages a town's defense team (spawning, deactivating, etc.)
//[town,position,side,unitTypes,deactivation time] Spawn AITownDefense
//*****************************************************************************************
Private["_camps","_center","_count","_defense","_defenses","_defensesOnly","_deactivationTime","_detectionRange","_enemies","_leader","_leaderType","_livingUnits","_location","_name","_objects","_opposingSide","_patrolRange","_position","_randomDefenses","_side","_soldierType","_team","_timeInactive","_total","_unit","_unitName","_unitTypes","_vehicles","_x"];
ScopeName "UpdateTownDefense";

_location = _this Select 0;
_position = _this Select 1;
_side = _this Select 2;
_unitTypes = _this Select 3;
_deactivationTime = _this Select 4;
_defenses = _this Select 5;
_defensesOnly = _this Select 6;

//Get town variables.
_name = Str _location;

_center = GetMarkerPos Format["%1CenterMarker",_name];
_detectionRange = Call Compile Format["%1Range",_name];
_patrolRange = _detectionRange * DETECTIONTOPATROLRANGERATIO;
_camps = Call Compile Format["%1Camps",_name];

_opposingSide = East;
_soldierType = WSOLDIER;
if (_side == East) then {_opposingSide = West;_soldierType = ESOLDIER};

_team = CreateGroup _side;

//If defense type is a vehicle then make the leader a soldier & move into vehicle.
_leaderType = _unitTypes Select 0;
if (!(_leaderType In soldierUnits) || _defensesOnly) then {_leaderType = _soldierType};

_leader = _team CreateUnit [_leaderType,_position,[],0.5,"FORM"];
_leader SetSkill 0.5;
[_leader,_position,50] Call PlaceSafe;
_position = GetPos _leader;
[_leader,_side] Call InitUnit;

if (!_defensesOnly) then
{
	_count = 1;
	if (!((_unitTypes Select 0) In soldierUnits)) then {_count = 0};

	_total = Count _unitTypes;
	for [{_count = _count},{_count < _total},{_count = _count + 1}] do
	{
		_unitName = _unitTypes Select _count;

		//Create vehicle.
		if (!(_unitName In soldierUnits)) then
		{
			_vehicle = _unitName CreateVehicle _position;
			[_vehicle,_side] Call InitUnit;

			//First slot is the leader so move into the vehicle.
			if (_count == 0) then
			{
				_leader MoveInDriver _vehicle;
			}
			else
			{
				_unit = _team CreateUnit [_soldierType,_position,[],0.5,"FORM"];
				_unit SetSkill 0.5;
				_unit MoveInDriver _vehicle;
				[_unit,_side] Call InitUnit;
			};

			if (_vehicle EmptyPositions "gunner" > 0) then
			{
				_unit = _team CreateUnit [_soldierType,_position,[],0.5,"FORM"];
				_unit SetSkill 0.5;
				_unit MoveInGunner _vehicle;
				[_unit,_side] Call InitUnit;
			};
		}
		//Create soldier.
		else
		{
			_unit = _team CreateUnit [_unitName,_position,[],0.5,"FORM"];
			_unit SetSkill 0.5;
			[_unit,_side] Call InitUnit;
		};
	};
};

_total = Count _defenses;

if (_total > 0) then
{
	//If there are more defenses than soldiers then randomly pick some.
	if (_total > MAXMANNEDTOWNDEFENSES) then
	{
		_total = MAXMANNEDTOWNDEFENSES;
		_randomDefenses = [];

		for [{_count = 0},{_count < _total},{_count = _count + 1}] do
		{
			_defense = _defenses Select (Random Count _defenses - 1);
			_randomDefenses = _randomDefenses + [_defense];
			_defenses = _defenses - [_defense];
		};

		_defenses = _randomDefenses;
	};

	_count = 0;

	//If only manning defenses then move leader into a defense.
	if (_defensesOnly) then
	{
		_leader MoveInGunner (_defenses Select 0);
		_count = 1;
	};

	//Create extra units for defenses.
	for [{_count = _count},{_count < _total},{_count = _count + 1}] do
	{
		_unit = _team CreateUnit [_soldierType,_position,[],0,"FORM"];
		_unit SetSkill 0.5;
		_unit MoveInGunner (_defenses Select _count);
		[_unit,_side] Call InitUnit;
	};
};

_timeInactive = time;

while {!IsNull _team} do
{
	if (!(_team Call IsTeamAlive)) then
	{
		DeleteGroup _team;
		_team = ObjNull;
	};

	if (time - _timeInactive >= _deactivationTime) then
	{
		if (!IsNull _team) then
		{
			//Move out of defenses.
			{if (Vehicle _x In _defenses) then {UnassignVehicle _x;_x Action ["EJECT",Vehicle _x]};} ForEach Units _team;
			Sleep 2;

			_vehicles = [_team,false] Call GetTeamVehicles;
			{DeleteVehicle _x} ForEach _vehicles;
			{DeleteVehicle _x} ForEach Units _team;
			DeleteGroup _team;
			_team = ObjNull;
		};
	};

	if (!IsNull _team) then
	{
		//Check for enemy activity.
		_objects = NearestObjects[_center,["Man","Air","Car","Motorcycle","Tank"],_detectionRange];
		_livingUnits = _objects Call GetLiveUnits;
		_enemies = (Resistance CountSide _livingUnits) + (_opposingSide CountSide _livingUnits);

		//If there are enemies reset the inactive timer.
		if (_enemies > 0) then
		{
			_timeInactive = time;
		};

		_locationSide = (Call Compile Format["%1SideID",_name]) Call GetSideFromID;

		//Retake town.
		if (_locationSide != _side) then
		{
			_team Move ([GetPos _location,7,20] Call GetRandomPosition);
		}
		//Patrol.
		else
		{
			//Patrol randomly around the town.
			if ((40 > Random 100) || Count _camps < 1) then
			{
				_position = [_center,0,_patrolRange] Call GetRandomPosition;
			}
			else
			{
				_position = [GetPos (_camps Select (Random (Count _camps - 1))),10,50] Call GetRandomPosition;
			};

			if (!SurfaceIsWater _position) then
			{
				_team Move _position;
			};
		};

		Sleep 30;
	};
};

//*****************************************************************************************
//1/13/8 MM - Created file.
